﻿using HarmonyLib;
using I2.Loc;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using static Tayx.Graphy.GraphyDebugger;

public class DirectionalSigns : Mod
{
    private Harmony harmony;
    public static AssetBundle assetbundle;
    public static GameObject tobject;

    public static GameObject menu;
    public static GameObject menu_asset;
    public static DirectionalSignsMenu directionalSignsMenu;

    public static Dictionary<uint, DirectionalSign_Network> signs = new Dictionary<uint, DirectionalSign_Network>();

    public void Start()
    {
        HNotification notification = FindObjectOfType<HNotify>().AddNotification(HNotify.NotificationType.spinning, "Loading DirectionalSigns ...");

        var bundleLoadRequest = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("directionalsigns.assets"));
        assetbundle = bundleLoadRequest.assetBundle;

        var language_en = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Signpost@A signpost for your raft." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Signpost" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Change the Signpost text and layout!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "How many signs?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotation" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Text" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Enter Text" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "en", Name = "English" } }
        };

        var language_de = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Wegweiser@Ein Wegweiser für dein Floß." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Wegweiser" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Wegweiser anpassen!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Wie viele Schilder?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Drehung" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Text" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Text eingeben" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "de", Name = "Deutsch" } }
        };

        var language_es = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Poste indicador@Un poste indicador para su balsa." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Poste indicador" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "¡Cambie el texto y el diseño de le poste indicador!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "¿Cuántos signos?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotación" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Texto" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Ingrese texto" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "es", Name = "Español" } }
        };

        var language_fr = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Poteau indicadeur@Un poteau indicadeur pour votre radeau." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Poteau indicadeur" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Modifiez le texte et la mise en page du poteau indicateur !" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Combien de signes ?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotation" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Texte" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Entrez du texte" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "fr", Name = "Français" } }
        };

        var language_se = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Vägisvare@En vägisvare för din flotte." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Vägvisare" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Ändra guidepostens text och layout!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Hur många tecken?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotation" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Text" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Skriv text" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "se", Name = "Svenska" } }
        };

        var language_it = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Guida@Una guida üer la tua zattera." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Guida" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Cambia il testo e il layout della guida!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Quanti segni?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotazione" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Testo" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Inserire il testo" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "it", Name = "Italiano" } }
        };

        var language_pl = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Drogowskaz@Drogowskaz dla Twojej tratwy." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Drogowskaz" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Zmień tekst i układ drogowskazu!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Ile znaków?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Obrót" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Tekst" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Wprowadź tekst" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "pl", Name = "Polski" } }
        };

        var language_pt = new LanguageSourceData()
        {
            mDictionary = new Dictionary<string, TermData>
            {
                ["philippr_directionalsign"] = new TermData() { Languages = new[] { "Guia@Um guia para sua jangada." } },
                ["CraftingSub/DirectionalSign"] = new TermData() { Languages = new[] { "Guia" } },
                ["DirectionalSign/Change"] = new TermData() { Languages = new[] { "Altere o texto e o layout da guia!" } },
                ["DirectionalSign/Count"] = new TermData() { Languages = new[] { "Quantos Signos?" } },
                ["DirectionalSign/Rotation"] = new TermData() { Languages = new[] { "Rotação" } },
                ["DirectionalSign/Text"] = new TermData() { Languages = new[] { "Texto" } },
                ["DirectionalSign/EnterText"] = new TermData() { Languages = new[] { "Digite o texto" } }
            },
            mLanguages = new List<LanguageData> { new LanguageData() { Code = "pt", Name = "Português-Brasil" } }
        };

        LocalizationManager.Sources.Add(language_en);
        LocalizationManager.Sources.Add(language_de);
        LocalizationManager.Sources.Add(language_es);
        LocalizationManager.Sources.Add(language_fr);
        LocalizationManager.Sources.Add(language_se);
        LocalizationManager.Sources.Add(language_it);
        LocalizationManager.Sources.Add(language_pl);
        LocalizationManager.Sources.Add(language_pt);

        Traverse.Create(typeof(LocalizationManager)).Field("OnLocalizeEvent").GetValue<LocalizationManager.OnLocalizeCallback>().Invoke();

        tobject = assetbundle.LoadAsset<GameObject>("DirectionalSign");

        var DirectionalSignBase = assetbundle.LoadAsset<Item_Base>("Directional_Sign");

        RAPI.AddItemToBlockQuadType(DirectionalSignBase, RBlockQuadType.quad_foundation);
        RAPI.AddItemToBlockQuadType(DirectionalSignBase, RBlockQuadType.quad_floor);
        RAPI.RegisterItem(DirectionalSignBase);

        foreach (Block b in DirectionalSignBase.settings_buildable.GetBlockPrefabs())
        {
            b.networkedBehaviour = b.gameObject.AddComponent<DirectionalSign_Network>();
            b.networkType = NetworkType.NetworkBehaviour;
        }

        DirectionalSignBase.set_recipe(new[]{new CostMultiple(new[] { ItemManager.GetItemByIndex(21) }, 5),new CostMultiple(new[] { ItemManager.GetItemByIndex(20) }, 5),new CostMultiple(new[] { ItemManager.GetItemByIndex(23) }, 4),new CostMultiple(new[] { ItemManager.GetItemByIndex(108) }, 5)}, CraftingCategory.Decorations, 1, false, "DirectionalSign", 1);

        var request_menu = assetbundle.LoadAssetAsync<GameObject>("DirectionalSignsMenu");
        menu_asset = request_menu.asset as GameObject;
        menu = Instantiate(menu_asset, this.transform);
        directionalSignsMenu = menu.AddComponent<DirectionalSignsMenu>();

        this.harmony = new Harmony("de.philippredel.DirectionalSigns");
        this.harmony.PatchAll(Assembly.GetExecutingAssembly());

        LocalizationManager.LocalizeAll();

        notification.Close();
        
    }

    public void OnModUnload()
    {
        assetbundle.Unload(true);
        this.harmony?.UnpatchAll("de.philippredel.DirectionalSigns");
        Debug.Log("Mod Signposts has been unloaded!");
    }

    public void Update()
    {
        if (RAPI.IsCurrentSceneMainMenu())
        {
            return;
        }
        MessageHandler handler = new MessageHandler();
        handler.ReadP2P_Channel_DirectionalSigns();
    }

    public override void WorldEvent_WorldUnloaded()
    {
        signs.Clear();
    }
}

[HarmonyPatch(typeof(RGD_Block), "RestoreBlock")]
public class Patch_DirectionalSignRestoreBlock
{
    public static void Postfix(ref RGD_Block __instance, Block block)
    {
        if (__instance.BlockIndex == 17123)
        {
            RGD_Storage rgd = __instance as RGD_Storage;
            DirectionalSign_Network dn = block.networkedBehaviour as DirectionalSign_Network;
            if (rgd != null && dn != null)
            {
                dn.RestoreData(rgd);
            }
        }
    }
}

[HarmonyPatch(typeof(Block), "GetBlockCreationData")]
public class Patch_BlockCreateSaveData
{
    public static bool Prefix(ref Block __instance, ref RGD_Block __result)
    {
        if (__instance.GetComponent<DirectionalSign_Network>() != null)
        {
            __result = __instance.GetComponent<DirectionalSign_Network>().Serialize_Save() as RGD_Block;
            return false;
        }
        return true;
    }
}

static class ExtentionMethods
{
    public static void set_recipe(this Item_Base item, CostMultiple[] cost, CraftingCategory category = CraftingCategory.Resources, int craftamount = 1, bool learned_from_beginning = false, string second_category = null, int second_category_order = 0)
    {
        Traverse recipe = Traverse.Create(item.settings_recipe);
        recipe.Field("craftingCategory").SetValue(category);
        recipe.Field("amountToCraft").SetValue(craftamount);
        recipe.Field("learnedFromBeginning").SetValue(learned_from_beginning);
        recipe.Field("subCategory").SetValue(second_category);
        recipe.Field("subCatergoryOrder").SetValue(second_category_order);
        item.settings_recipe.NewCost = cost;
    }
}